// Line coding examples - 4B5B
// R.Lacoste / June 2011 - Circuit Cellar #255
// Developped under Scilab 5.3.2

// Constants
Nb_input_bits=4000;               // Number of transmitted bits, multiple of 4
dt=1e-6;                          //1Msps
Nb_exp_factor=10;                // Number of calculation steps per bit
Nb_exp_factor5=8;                // Number of expension steps per bit after 4B5B substitution
Nb_plotted_bits=12;                // Number of bits displayed on-screen

// Generate input bits
rnd=grand(1, Nb_input_bits, 'def');    // random values from 0 to 1
input_bits=(rnd>0.5);                  // true/false bits
input_bits(1:20)=[0 1 0 0 0 1 1 1 1 0 1 0 0 0 0 1 0 1 1 0]; // For easy comparison of first bits
rep=(1:Nb_exp_factor);
rep(:)=1;
expended_bits=kron(input_bits,rep);    // Duplicate each input bit 10 times
time_scale=((1:Nb_exp_factor*Nb_input_bits)-1)*dt/Nb_exp_factor;
subplot(3,1,1);
plot2d(time_scale(1:Nb_plotted_bits*Nb_exp_factor),expended_bits(1:Nb_plotted_bits*Nb_exp_factor),rect=[0,-0.1,Nb_plotted_bits*dt,1.1],nax=[1,Nb_plotted_bits+1,1,7]);
a=gca(); // get the handle of the current axes
a.grid=[1 -1 -1]; //make x-grid

// Channel coding
exp4b5b_bits=1:(5*Nb_input_bits/4);
for i=1:Nb_input_bits/4 do
    substring=input_bits(1+(i-1)*4:4+(i-1)*4);
    if (substring==[0 0 0 0]) then
        outsubstring=[1 1 1 1 -1];
    elseif (substring==[0 0 0 1]) then
        outsubstring=[-1 1 -1 -1 1];
    elseif (substring==[0 0 1 0]) then
        outsubstring=[1 -1 1 -1 -1];
    elseif (substring==[0 0 1 1]) then
        outsubstring=[1 -1 1 -1 1];
    elseif (substring==[0 1 0 0]) then
        outsubstring=[-1 1 -1 1 -1];
    elseif (substring==[0 1 0 1]) then
        outsubstring=[-1 1 -1 1 1];
    elseif (substring==[0 1 1 0]) then
        outsubstring=[-1 1 1 1 -1];
    elseif (substring==[0 1 1 1]) then
        outsubstring=[-1 1 1 1 1];
    elseif (substring==[1 0 0 0]) then
        outsubstring=[1 -1 -1 1 -1];
    elseif (substring==[1 0 0 1]) then
        outsubstring=[1 -1 -1 1 1];
    elseif (substring==[1 0 1 0]) then
        outsubstring=[1 -1 1 1 -1];
    elseif (substring==[1 0 1 1]) then
        outsubstring=[1 -1 1 1 1];
    elseif (substring==[1 1 0 0]) then
        outsubstring=[1 1 -1 1 0];
    elseif (substring==[1 1 0 1]) then
        outsubstring=[1 1 -1 1 1];
    elseif (substring==[1 1 1 0]) then
        outsubstring=[1 1 1 -1 -1];
    elseif (substring==[1 1 1 1]) then
        outsubstring=[1 1 1 -1 1];
    end;
    exp4b5b_bits(1+(i-1)*5:5+(i-1)*5)=outsubstring;
end;

rep=(1:Nb_exp_factor5);
rep(:)=1;
signal=kron(exp4b5b_bits,rep); 

subplot(3,1,2);
plot2d(time_scale(1:Nb_plotted_bits*Nb_exp_factor),signal(1:Nb_plotted_bits*Nb_exp_factor),rect=[0,-1.5,Nb_plotted_bits*dt,1.5],nax=[1,Nb_plotted_bits+1,1,7]);
a=gca(); // get the handle of the current axes
a.grid=[1 -1 -1]; //make x-grid

// Calculate and plot its power spectrum

// Calculate its Fourier tranform, giving complex numbers
spectrum=fft(signal);

// However as the input signal is real then only the first half of the FFT
// is actually useful (the last half is the complex conjugate of the first half) :
usefulspectrum=spectrum(1:$/2+1);

// Calculate amplitudes and phases of all frequency bins
amplitudes=abs(usefulspectrum); 
frequencies=((1:Nb_exp_factor*Nb_input_bits/2+1)-1)/(Nb_input_bits*dt);
subplot(3,1,3);
plot(frequencies,amplitudes,nax=[1,11,1,7])
a=gca(); // get the handle of the current axes
a.grid=[1 -1 -1]; //make x-grid
