/********************************************************/
/* CupWarmer.C
/* I/A = Input Analog
// O/D = Output Digital
// Pin		State		Description
// RA0:		I/A		Coffee cup weight
// RA1:		O/D		Heater on signal to triac
// RA2:		O/D		LCD RS
// RA3:		I/D		Button - Calibrate for new coffee cup
// RA4:		O/D		LCD E
// RA5:		I/D		Thermocouple wire alarm
// RC0:		I/A		Temperature Control Knob
// RC1:		I/A		Temperature
// RC2-5	O/D		LCD data
// RA1 PWM period=1minute, pulse width set by PWM value at 1 second resolution

// EEPROM Data
// Address h	State	Description
//	0			rw		Cupweight Low byte
//	1			rw		Cupweight High byte

/********************************************************/

// Includes
#include 	<pic16f684.h>
#include	<pic.h>
#include	<delay.h>
#include	<lcd.h>
#include	<CupWarmer.h>
//_CONFIG(UNPROTECT & BORDIS & PWRTDIS & WDTDIS & INTRC_OSC_NOCLKOUT & MCLRDIS & FCMDIS & IESODIS);
// Variables
unsigned char State;				// State Machine
unsigned char PWM;				// PWM width goes from 0 to 60, period = 1 sec.
unsigned char Timer3;				// Short timer, counts to 1 second
char String[4];					// LED display string
unsigned int Timer4;				// Long timer	, 1800 secs = 30 minutes
unsigned int heater;				// heater input temp
unsigned int knob;				// Temp control knob setting
unsigned int Cweight;				// Coffe cup weight
unsigned int Temperature;			// Converted Temperature
int PIDout;

void main()
{	IRCF2 =1;				// Internal oscillator at4 MHz
	IRCF1 = 1;
	IRCF0 = 0;
	SCS = 1;				// Internal oscillator as system clock
	CMCON0 = 0b00000100;			// Turn on Comparators, 2 inputs, no interrupt
	VRCON = 0b00100110;			// Set and enable Vref off
	ANSEL = 0b00110001;			// Set analog inputs. 
	ADCON1 = 0b01110000;			// A/D clock = RC 
	TRISA = 0b00101001;			// Set PortA I/O
	IOCA = 0b00101000;			// Interrupt on change for RA3, RA5
	TRISC = 0b00000011;			// Set PortC I/O: 
	OPTION = 0b00000111;			// TMR0, internal, 256 prescaler
	TMR0 = 0x0BF;				// Timer0, 0x0DF = 1/60s, 0x0BF = 1/30s
	RAIE = 1;				// Enable Interrupt on change for PortA
	T0IE = 1;				// Enable Timer0 interrupt
	GIE = 1;				// Enable Global Interrupts
	PEIE = 1;				// Enable Peripheral interrupts
	Timer3 = 0;				// Make sure timers are zero
	Timer4=0;
	PWM = 0;				// Default PWM value
	State = 0;
	PORTA = 0;

	lcd_init();				// Initialize LCD
	lcd_clear();				// Clear and set cursor home
	lcd_puts("Hello");
	lcd_goto(0x40);				// move cursor to next line on display
	lcd_puts("Carl");

	for(;;)								// Main Loop
	{	 CLRWDT();						// kick the dog
		if(State==1)	
		{	control();
		}
		if(State == 2)
		{	State++;
			read_cup();					// Read cup weight	
		}
		if(State == 3)						// Take settings
		{	State++;
			heater = heater/2;
			heater = heater + adc_read(Theater)/2;		// Read A/D, ANS5=RC1
			knob = knob/ 2;					// divide by 2 as an averageing filter
			knob = knob + adc_read(Tcontrol)/2;		// Read knob T setting ANS4 = RC0
			PIDout = PID(knob-heater, heater);		// knob - heater = error
		}				
		if(State == 4)						// Display 
		{	State++;					// incrementstate machine
			lcd_clear();
			lcd_puts("S: ");
			convert(knob);					// Convert and display
			lcd_goto(0x40);					// move cursor to next line on display
			lcd_puts("T: ");
			convert(heater);				//Conversion to F
			//lcd_clear();
			//itoa10(String,PIDout);
			//lcd_puts(String);
		}
		if(State == 5)						// Set PWM bounds
		{	State = 1;
			if(PIDout<0)
				PWM = 0;
			else if(PIDout > 60)
				PWM = 60;
			else PWM = PIDout;				// Set PWM width
		}
		if(State == 7)						// Alarm.  The thermocouple wire is disconnected
		{	lcd_clear();
			lcd_puts("Sensor");				// or broken.
			lcd_goto(0x40);					// Move cursor to next line on display
			lcd_puts("Error!");
			PORTA = 0;	
			State = 0;					// Disable state
			GIE=0;						// Disable interrupts
		}
		if(State == 8)						// Save Cup weight in eeprom
		{	calibrate_cup();
			PORTA = 0;	
			State =1;
		}
	}  // End for
} // End Main

void convert(unsigned int Tin)
{	char String[4];
	unsigned int temp;
	temp = ((Tin * 87/100) + 32);				// Conversion to F
	itoa10(String,temp);
	lcd_puts(String);
}

int PID(int error, unsigned int T)
{	int pTerm;						// Everything is in terms of bits per volt
	int	iTerm;
	int  dTerm;
	static int iState;
	static unsigned int TState;
	pTerm =error/ pGain;					// Proportional term
	iState = iState  + error;				//read in last iState value and update
	if(iState > iMax)					//check for max
		iState = iMax;
	else if(iState < iMin)					//check for min
		iState = iMin;		
	iTerm = iState/iGain ;					// Integral term.  NOTE divisor to avoid using float value	
	dTerm = dGain *  (T - TState);				// Differential term, present - last
	TState = T;						// Temperature update for next iteration
	return (pTerm + iTerm - dTerm);				// return scaled control value
}

unsigned int adc_read(unsigned char channel)
{	ADCON0 = (channel << 2) + 0x81;				// enable ADC.	
	ADIF = 0;						// Clear interrupt flag
	GODONE = 1;						// Start A/D
	while(!ADIF)
		continue;					// wait for conversion complete
	ADON = 0;						// turn off A/D Module
	return (ADRESH <<8) + ADRESL ;				// Shift in value
}

void read_cup(void)
{	unsigned int temp;
	Cweight = adc_read(Cup);				// Read Cup
	temp =  eeprom_read(cupH)<<8;
	temp = temp + eeprom_read(cupL);			// get stored calibrated cup weight	
	if(Cweight<temp)
	{	Timer4=0;					// Reset timer, no cup on heater
		sleepnow();					// sleep
	}
	else if (Timer4>1800)
	{	sleepnow();					// Timed out, sleep
		Timer4=1800;					// Prevent overrun
	}
	return;
}

void sleepnow(void)
{		State = 0;					// reset State
		PORTA = 0;					// Turn off heater
		lcd_clear();					// Clear display
		WDTCON = 0b00010111;				// Turn WDT on, sleep for .47 secs
		lcd_write(0x08);				// Display off
		CLRWDT();					// Clear WDT for interrupt wake up
		asm("sleep");
		asm("NOP");
		lcd_write(0x0C);				// Display on
		WDTCON = 0b00010110;				// Turn WDT off
}

void calibrate_cup(void)
{	lcd_clear();
	lcd_puts("Weighing");
	lcd_goto(0x40);						//move cursor to next line on display
	lcd_puts("Cup:");
	DelayMs(255);						// Bunch of delays to let the user view reading
	DelayMs(255);
	DelayMs(255);
	DelayMs(255);
	DelayMs(255);
	Cweight = adc_read(Cup);				// Read A/D, ANS0=RA0
	eeprom_write(cupL, Cweight);				// Store value in EEPROM
	eeprom_write(cupH, Cweight>>8);
	return;
}

// RA1 PWM period=1minute, pulse width set by PWM value at 1 second resolution
void control(void)						// Turn Triac on and off 
{	if(Timer3 < PWM)					// Check PWM width and set point temp from comparator2
	{	if(C2OUT)	PORTA = 2;			// RA1 is the controlling pin, turn on
	}
	else
	{	PORTA = 0;					// Turn off
		State = 0;					// Reset state machine
	}
	C2IF = 0;						// Clear Comparator 1 interrupt
	C1IF = 0;						// Clear Comparator 2 interrupt
	return;
}

static void interrupt isr(void)					// Interrupt function
{	if(T0IF)						// Timer0 overflow interrupt
	{ 	TMR0 = 0x0BF;					// Reset Timer0
		Timer3 ++;					// increment short timer
		if(Timer3 > 59)					// Have we ticked off one second?
		{	Timer3=0;				// Yes - Reset Timer3		
			Timer4++;				// increment long timer
			State = 2;				// Update state once a second
		}
		T0IF = 0;					// Reset TMR0
	}
	
	if(RAIF)						// Check for button interrupt
	{	if(!RA3)		State = 8;		// Calibrate cup
		else if(RA5)	State = 7;			// Alarm, thermocouple wire disconnected
		RAIF = 0;					// Clear Interrupt		
	}
} // End isr 


