// Line coding examples - NRZI
// R.Lacoste / June 2011 - Circuit Cellar #255
// Developped under Scilab 5.3.2

// Constants
Nb_input_bits=5000;               // Number of transmitted bits
dt=1e-6;                          //1Msps
Nb_exp_factor=10;                // Number of calculation steps per bit
Nb_plotted_bits=12;                // Number of bits displayed on-screen

// Generate input bits
rnd=grand(1, Nb_input_bits, 'def');    // random values from 0 to 1
input_bits=(rnd>0.5);                  // true/false bits
input_bits(1:20)=[0 1 0 0 0 1 1 1 1 0 1 0 0 0 0 1 0 1 1 0]; // For easy comparison of first bits
rep=(1:Nb_exp_factor);
rep(:)=1;
expended_bits=kron(input_bits,rep);    // Duplicate each input bit 10 times
time_scale=((1:Nb_exp_factor*Nb_input_bits)-1)*dt/Nb_exp_factor;
subplot(3,1,1);
plot2d(time_scale(1:Nb_plotted_bits*Nb_exp_factor),expended_bits(1:Nb_plotted_bits*Nb_exp_factor),rect=[0,-0.1,Nb_plotted_bits*dt,1.1],nax=[1,Nb_plotted_bits+1,1,7]);
a=gca(); // get the handle of the current axes
a.grid=[1 -1 -1]; //make x-grid

// Channel coding
nrzi=input_bits;
nrzi(1)=0;
for i=2:Nb_input_bits do
    if(input_bits(i)==1) then
        nrzi(i)=1-nrzi(i-1);
    else
        nrzi(i)=nrzi(i-1)
    end;
end;
expended_bits=kron(nrzi,rep);    // Duplicate each input bit 10 times
signal=2*bool2s(expended_bits)-1;      // bit0=-1V, bit1=+1V
subplot(3,1,2);
plot2d(time_scale(1:Nb_plotted_bits*Nb_exp_factor),signal(1:Nb_plotted_bits*Nb_exp_factor),rect=[0,-1.5,Nb_plotted_bits*dt,1.5],nax=[1,Nb_plotted_bits+1,1,7]);
a=gca(); // get the handle of the current axes
a.grid=[1 -1 -1]; //make x-grid

// Calculate and plot its power spectrum

// Calculate its Fourier tranform, giving complex numbers
spectrum=fft(signal);

// However as the input signal is real then only the first half of the FFT
// is actually useful (the last half is the complex conjugate of the first half) :
usefulspectrum=spectrum(1:$/2+1);

// Calculate amplitudes and phases of all frequency bins
amplitudes=abs(usefulspectrum);
frequencies=((1:Nb_exp_factor*Nb_input_bits/2+1)-1)/(Nb_input_bits*dt);
subplot(3,1,3);
plot(frequencies,amplitudes,nax=[1,11,1,7])
a=gca(); // get the handle of the current axes
a.grid=[1 -1 -1]; //make x-grid
